import React from 'react'
import PropTypes from 'prop-types'

export default class CheckBox extends React.Component {

  static propTypes = {
    onChange: PropTypes.func,
    checked: PropTypes.bool,
    disabled: PropTypes.bool,
    label: PropTypes.string
  };

  constructor(props) {
    super(props)
    this.checked = props.checked;

    this.changeValue = (e) => {
      this.checked = e.currentTarget.checked;
      this.setState({
        checked: this.checked
      });
      this.props.onChange && this.props.onChange(this.checked);
    }
  }

  componentWillReceiveProps(props) {
    if (props.checked !== undefined) {
      this.checked = props.checked
    }
  }


  render() {
    let {label, disabled} = this.props;

    return <label className={"checkbox-wrapper"+(disabled?" checkbox-disabled":"")}>
      <input type="checkbox" checked={this.checked} disabled={disabled}
             onChange={this.changeValue}/>
      {label}
    </label>
  }
}

export class CheckBoxGroup extends React.Component {

  static defaultProps = {
    disabled: [],
    checked: [],
    options: [],
  }

  static propTypes = {
    onChange: PropTypes.func,
    checked: PropTypes.array,
    defaultChecked: PropTypes.array,
    options: PropTypes.array,
    disabled: PropTypes.array
  };

  constructor(props) {
    super(props)
    this.checked = props.checked;

    this.changeValue = (e, value) => {
      let checked = e.currentTarget.checked;
      if (checked) {
        this.checked.push(value)
      }else {
        this.checked.splice(this.checked.indexOf(value),1)
      }
      this.setState({
        checked: this.checked
      });
      this.props.onChange && this.props.onChange(this.checked);
    }
  }

  componentWillReceiveProps(props) {
    if (props.checked !== undefined) {
      this.checked = props.checked
    }
  }

  render() {
    let {options, disabled} = this.props;

    return <div>
      {
        options.map((item, i) => {
          return <label className={"checkbox-wrapper"+(disabled.indexOf(item.value) >= 0?" checkbox-disabled":"")} key={i}>
            <input type="checkbox" checked={this.checked.indexOf(item.value) >= 0}
                   disabled={disabled.indexOf(item.value) >= 0} ref="checkbox" onChange={(e) => {
              this.changeValue(e, item.value)
            }}/>
            {item.label}
          </label>
        })
      }
    </div>
  }
}