import React from 'react'
import PropTypes from 'prop-types'

export default class InputNumber extends React.Component {
  static defaultProps = {
    min: 0
  }

  static propTypes = {
    disabled: PropTypes.bool,
    defaultValue: PropTypes.number,
    min: PropTypes.number,
    max: PropTypes.number,
    step: PropTypes.number,
    onChange: PropTypes.func,
    onKeyDown: PropTypes.func,
    onBlur: PropTypes.func,
    value: PropTypes.number,
  };

  constructor(props) {
    super(props);
    let value = props.value;
    if (value === undefined && props.defaultValue !== undefined){
      value = props.defaultValue
    } else if (value === undefined){
      value = null
    }
    this.state = {
      hover: false,
      value: value,
      preValue: value,
    }
    this.value = value;
  }

  componentWillReceiveProps(props) {
    if (props.value !== undefined && props.value != this.value) {
      this.value = props.value;
      this.refs['input'].value = this.value;
      this.setState({
        value: this.value
      })
    }
  }

  count(type) {
    var {min, max, step} = this.props;
    step = parseInt(step || 1);
    var value = this.state.value || 0;


    value = parseInt(value);
    if (type == '+') {
      value = value + step;
    } else if (type == '-') {
      value = value - step;
    }


    if ((min || min === 0) && value < min) {
      value = min;
    }
    if (max && value > max) {
      value = max;
    }

    this.setState({
      value
    })
    this.value = value;
    this.props.onChange && this.props.onChange(value)
  }

  onKeyDown(e) {
    switch (e.key) {
      case '+':
      case'ArrowUp':
        this.count('+')
        break;
      case '-':
      case 'ArrowDown':
        this.count('-')
        break;
    }
    this.props.onKeyDown && this.props.onKeyDown(e.which)
  }

  onChange(e) {
    let value = this.refs['input'].value;
    value = this.getValue(value);
    this.props.onChange && this.props.onChange(value)
  }

  blur() {
    let value = this.getValue()
    this.props.onBlur && this.props.onBlur(value)
  }

  getValue(value) {
    if (value === undefined) {
      value = this.state.value;
    }

    var {preValue} = this.state;
    var {max, min} = this.props;
    value = parseInt(value);

    if (isNaN(value)) {
      value = ""
    } else if ((min || min === 0) && value < min) {
      value = min;
      preValue = value
    } else if (max && value > max) {
      value = max;
      preValue = value
    } else {
      preValue = value
    }
    this.setState({
      value,
      preValue
    })
    this.value = value;
    return value;
  }

  render() {
    var {hover, value} = this.state;
    return (
      <div className="number-control"
           onMouseOver={() => {
             this.setState({hover: true})
           }}
           onMouseLeave={() => {
             this.setState({hover: false})
           }}
           onKeyDown={(e) => {
             this.onKeyDown(e)
           }}
      >
        <input type="text" ref="input" className="input-number"
               {...this.props} value={value} onChange={(e) => {
          this.onChange(e)
        }}
               onBlur={() => {
                 this.blur()
               }}
        />
        {hover ?
          <div className="number-control-icon">
            <span className="number-control-icon-item icon-formup" onClick={() => {
              this.count('+')
            }}></span>
            <span className="number-control-icon-item icon-formdown" onClick={() => {
              this.count('-')
            }}></span>
          </div> : null
        }
      </div>
    )
  }
}
