import React from 'react'
import PropTypes from 'prop-types'

export default class Menu extends React.Component {
  
  static defaultProps = {
    multiple:false
  }

  static propTypes = {
    /**
     {
        title:"main",
        href:"/main",
        key:"",
        list:[
          { 
            title:"sub",
            href:"sub",
            key:""
          }
        ]
     } 
    */
    dataSource:PropTypes.array.isRequired,
    openKeys:PropTypes.array,
    defaultOpenKeys:PropTypes.array, //[key1,key2]
    selectedKeys:PropTypes.array,
    defaultSelectedKeys:PropTypes.array,
    onSelect:PropTypes.func, //选中后的回调
    multiple:PropTypes.bool
  };

  constructor(props) {
    super(props);

    this.state = {
      selectedKeys:props.selectedKeys || props.defaultSelectedKeys || [],
      openKeys:props.openKeys || props.defaultOpenKeys || []
    }
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      selectedKeys:nextProps.selectedKeys || this.state.selectedKeys,
      openKeys:nextProps.openKeys || this.state.openKeys
    })
  }

  onOpenKeys(key,menu, e){

    if(key){
      let openKeys = this.state.openKeys.slice(0);
      if(openKeys.indexOf(key) != -1){
        openKeys.splice(openKeys.indexOf(key), 1);
      }else{
        openKeys.push(key);
      }
      this.setState({
        openKeys:openKeys
      })
    } 
  }
  
  onSelectedKey(key, menu, e){
    if(key){
      let selectedKeys = this.state.selectedKeys.slice(0);
      if(selectedKeys.indexOf(key) != -1){
        selectedKeys.splice(selectedKeys.indexOf(key), 1);
      }else{
        selectedKeys = [key];
      }
      this.setState({
        selectedKeys:selectedKeys
      })
    }
    //调用 onSelect
    this.props.onSelect && this.props.onSelect(menu, e);
  }

  renderItem(menu){
    return (
      <a href={menu.href || "javascript:;"} onClick={menu.list ? this.onOpenKeys.bind(this, menu.key, menu) : this.onSelectedKey.bind(this,menu.key, menu)} >
        {menu.title}
        {menu.list ? <span className="icon-caret icon-caret-color"></span> : null}
      </a>
    )
  }

  renderSubItem(item, index){
    let {selectedKeys} = this.state;

    return <li className={selectedKeys.indexOf(item.key) != -1 ? "list-dropdown-item list-action" : "list-dropdown-item"} key={index}>
            <a href={item.href || "javascript:;"} onClick={this.onSelectedKey.bind(this, item.key, item)} >{item.title}</a>
          </li>
  }
  //判断grouplist的class
  getListGroupClassName(menu){
    let openKeys = this.state.openKeys;
    let selectedKeys = this.state.selectedKeys;
    let isChildList = !!(menu.list && menu.list.length);
    let isChecked = selectedKeys.indexOf(menu.key) != -1;

    if( isChildList && openKeys.indexOf(menu.key) != -1 ){ //(有子元素 && 被选中)
      return "list-group-item list-dropdown list-dropdown-action" //展开菜单
    }else if(isChecked){//没有子元素被选中
      return "list-group-item list-action"
    }else{
      return  "list-group-item"
    }
  }
  render(){

    let {dataSource} = this.props;
    let {openKeys} = this.state;
    //list-dropdown-action
    //list-action
    return (
      <ul className="yt-list-group">
        {
          dataSource.map((menu, index)=>{
            return (
              <li 
                className={this.getListGroupClassName(menu)} 
                key={index}
              >
                {this.renderItem(menu)}
                <ul className="dropdown-menu">
                  {
                    menu.list && menu.list.map((item, index)=>{
                      return this.renderSubItem(item, index)
                    })
                  }
                </ul>
              </li>
            )
          })
        }
      </ul>
    )
  }

}


