import React from 'react'
import PropTypes from 'prop-types'

import {render, unmountComponentAtNode} from 'react-dom';

const GlobalMessage = (props) => {
	const {type, content} = props;
	return (
		<div className="message-notice">
			<div className="message-notice-content">
				<span className={"message-icon "+type}/>
				<span>{content}</span>
			</div>
		</div>
	);
};

GlobalMessage.propTypes = {
	type: PropTypes.string.isRequired,
	content: PropTypes.string.isRequired
};

export default class Message {
	static success(content, timeout, onClose) {
		this.message('pirosicon-solid-success', content, timeout, onClose);
	}

	static info(content, timeout, onClose) {
		this.message('pirosicon-solid-info', content, timeout, onClose);
	}

	static warning(content, timeout, onClose) {
		this.message('pirosicon-solid-warning', content, timeout, onClose);
	}

	static error(content, timeout, onClose) {
		this.message('pirosicon-solid-error', content, timeout, onClose);
	}

	static loading(content, timeout, onClose){
		this.message('loading', content, timeout, onClose);
	}

	static message(type, content, timeout = 3000, onClose) {
		this.propTypes = {
			onClose:PropTypes.func,
		};
		const container = document.createElement('div');
		container.setAttribute('class', 'message message-action');
		document.body.appendChild(container);
		render(<GlobalMessage type={type} content={content} />, container);
		setTimeout(() => {
			unmountComponentAtNode(container);
			container.remove();
			onClose&&onClose();
		}, timeout);
	}

}