import React from 'react'
import PropTypes from 'prop-types'
import Button from './Button'

/**

 <Dialog title="" onOk={} onCancel={} okButton={true} cancelButton={true} >

 **/
export default class Modal extends React.Component {
  static defaultProps = {
    cancelButton: true,
    onButton: true,
    loading: false,
    okButtonText: "确定",
    cancelButtonText: "取消",
    title: "提醒",
    display: false,
    maskClick: false,
    enterClick: false
  }

  static propTypes = {
    display: PropTypes.bool,
    onOk: PropTypes.func,
    onCancel: PropTypes.func,
    title: PropTypes.string,
    loading: PropTypes.bool,
    okButtonText: PropTypes.string,
    onButton: PropTypes.bool,
    cancelButtonText: PropTypes.string,
    cancelButton: PropTypes.bool,
    footer: PropTypes.any,
    style: PropTypes.object,
    maskClick: PropTypes.bool,
    onHide: PropTypes.func
  };

  constructor(props) {
    super(props);

    this.state = {
      loading: false
    };

    this.onCancel = (e) => {
      this._onCancel(e)
      e.stopPropagation();
      return false;
    }

    this.onOk = (e) => {
      this._onOk(e)
    }

    this.maskClick = (e) => {
      if (e.target.className.indexOf("modal-action") > -1) {
        this._maskClick(e)
      }
    }

    this.keyDown = (e) => {
      if (this.props.display) {
        console.log(e)
        if (e.key === "Enter") {
          this._onOk()
        } else if (e.key === "Escape") {
          this._onCancel()
        }

      }
    }

    this.setParams(props);
  }

  componentWillMount() {
    if (this.props.enterClick) {
      document.addEventListener("keydown", this.keyDown)
    }
  }

  componentWillUnmount() {
    if (this.props.enterClick) {
      document.removeEventListener("keydown", this.keyDown)
    }
  }

  componentWillReceiveProps(props) {
    this.setParams(props);
  }

  setParams(props) {
    this.state.display = props.display;
    this.state.title = props.title;
    this.state.loading = props.loading;
    this.state.okButtonText = props.okButtonText;
    this.state.cancelButtonText = props.cancelButtonText;
    this.state.onOk = props.onOk;
    this.state.onCancel = props.onCancel;
  }

  _onOk() {
    //如果正在加载中，不允许重复点击
    if (this.state.loading) {
      return;
    }
    if (this.props.onOk) {
      this.props.onOk()
    }
  }

  _onCancel() {
    if (this.props.onCancel) {
      this.props.onCancel();
    } else {
      this.setState({
        display: false
      })
    }
  }

  _maskClick(e) {
    if (this.state.loading || !this.props.maskClick) {
      return;
    }

    this._onCancel(e)
  }

  render() {

    let {children, cancelButton, onButton, footer, style, enterClick} = this.props;
    let {okButtonText, cancelButtonText, title, display, loading, width} = this.state;
    let enterHover = {boxShadow: "0px 0px 0px 4px #C3DAFF"};
    if (footer === undefined) {
      footer = <div className="modal-footer">
        {
          cancelButton ?
            <Button onClick={this.onCancel} disabled={loading} secondary={true}>{cancelButtonText}</Button> : null
        }
        {
          onButton ?
            <Button style={enterClick ? enterHover : {}} onClick={this.onOk} disabled={loading}
                    loading={loading}>{okButtonText}</Button> : null
        }
      </div>
    }

    if (style === undefined) {
      style = {marginBottom: 50}
    } else {
      style.marginBottom = 50
    }
    let content = <div className="modal-content " style={style}>
      <span className="modal-close" onClick={this.onCancel}/>
      <div className="modal-header">{title}</div>
      <div className="modal-body">
        { children }
      </div>
      {footer}
    </div>

    return <div className={display ? "modal modal-action" : "modal"} onClick={this.maskClick}>
      {content}
    </div>
  }
}

