import React from 'react'
import PropTypes from 'prop-types'

export default class Pagination extends React.Component {

  static propTypes = {
    onChange: PropTypes.func.isRequired,
    total: PropTypes.number.isRequired,
    current: PropTypes.number,
    defaultCurrent: PropTypes.number,
    pageSize: PropTypes.number,
    maxShowPage: PropTypes.number,
  };

  constructor(props) {
    super(props)
    this.defaultSize = 10;
    this.state = {
      current: props.current || props.defaultCurrent || 1,
      countPage: Math.ceil((props.total || 0) / (props.pageSize || this.defaultSize)),
      total: props.total || 0,
      maxShowPage: props.maxShowPage || 5,
      hideJumper: props.hideJumper !== undefined || false
    };
  }

  componentWillReceiveProps(props) {
    let countPage = Math.ceil((props.total || 0) / (props.pageSize || this.defaultSize));
    let data = {
      countPage: countPage,
      total: props.total || 0,
      maxShowPage: props.maxShowPage || 5
    };
    if (props.current) {
      data.current = props.current;
    }
    this.setState(data)
  }

  toPageIndex(index) {
    index = parseInt(index)
    let {countPage, current} = this.state;
    let {pageSize} = this.props;
    if (1 <= index && index <= countPage) {
      this.setState({
        current: index
      });
      this.props.onChange(index, pageSize || this.defaultSize)
    } else {
      this.refs.goIndex.value = current;
    }
  }

  render() {
    let {current, countPage, total, maxShowPage, hideJumper} = this.state;
    let pages = [current];

    // 当前页分别向两侧增加两页
    for (let i = 1; i <= Math.floor(maxShowPage / 2); i++) {
      if (current - i >= 1) pages.unshift(current - i);
      if (current + i <= countPage) pages.push(current + i);
    }
    // 长度小于5,且第一页为1,最后一页小于总页数
    while (pages.length < 5 && pages[0] == 1 && pages[pages.length - 1] < countPage) {
      pages.push(pages[pages.length - 1] + 1)
    }

    // 长度小于5,且最后一页等于总页数,第一页大于1
    while (pages.length < 5 && pages[pages.length - 1] == countPage && pages[0] > 1) {
      pages.unshift(pages[0] - 1)
    }


    if (pages[0] > 3) {
      pages.unshift(".");
      pages.unshift(1);
    } else {
      while (pages[0] > 1) {
        pages.unshift(pages[0] - 1)
      }
    }

    if (pages[pages.length - 1] <= countPage - 3) {
      pages.push(".");
      pages.push(countPage);
    } else {
      while (pages[pages.length - 1] < countPage) {
        pages.push(pages[pages.length - 1] + 1)
      }
    }

    if (!hideJumper && countPage < 5) {
      hideJumper = true
    }

    return (
      <div className="pagination-wrapper con-wrapper" style={{marginBottom: 0, paddingBottom: "20px"}}>
        <span className="pagination-total-text">共 {total} 条</span>
        <ul className="pagination">
          <li className="pagination-prev" onClick={this.toPageIndex.bind(this, current - 1)}/>
          {
            pages.map((page, i) => {
              if (page == ".") {
                return <li className="pagination-ellipsis" key={i}/>
              } else {
                return <li key={i} onClick={this.toPageIndex.bind(this, page)}
                           className={page == current ? "pagination-item pagination-item-action" : "pagination-item"}>
                  <a>{page}</a></li>
              }
            })
          }
          <li className="pagination-next" onClick={this.toPageIndex.bind(this, current + 1)}/>
        </ul>
        {hideJumper ? null : <div className="pagination-options-quick-jumper">
          跳至 <input type="text" name="" className="input" ref="goIndex" onKeyPress={(e) => {
          if (e.key == "Enter") this.toPageIndex(e.target.value)
        }}/> 页
        </div>}
      </div>
    )
  }
}

