import React from 'react'
import PropTypes from 'prop-types'
import {renderToString} from 'react-dom/server';

export default class Popconfirm extends React.Component {

  static defaultProps = {
    direction: "top",
    visible: false
  };

  static propTypes = {
    direction: PropTypes.string,
    title: PropTypes.string,
    content: PropTypes.object,
    visible: PropTypes.bool
  };

  wrapper = null;
  popconfirm = null;

  constructor(props) {
    super(props);
    const inner = renderToString(this.props.content);
  }

  getElementLeft(element) {
    let actualLeft = element.offsetLeft;
    let current = element.offsetParent;
    while (current !== null) {
      actualLeft += current.offsetLeft;
      current = current.offsetParent;
    }
    return actualLeft;
  }

  getElementTop(element) {
    let actualTop = element.offsetTop;
    let current = element.offsetParent;
    while (current !== null) {
      actualTop += current.offsetTop;
      current = current.offsetParent;
    }
    return actualTop;
  }

  setPosition() {
    const wrapperStyle = getComputedStyle(this.wrapper, null);
    const popconfirmStyle = getComputedStyle(this.popconfirm, null);

    const wrapperWidth = parseFloat(wrapperStyle.width);
    const wrapperHeight = parseFloat(wrapperStyle.height);
    const popconfirmWidth = parseFloat(popconfirmStyle.width);
    const popconfirmHeight = parseFloat(popconfirmStyle.height);

    const wrapperTop = this.getElementTop(this.wrapper);
    const wrapperLeft = this.getElementLeft(this.wrapper);

    switch (this.props.direction) {
      case 'top':
        this.popconfirm.style.left = wrapperLeft + wrapperWidth / 2 + 'px';
        this.popconfirm.style.top = wrapperTop + 'px';
        break;
      case 'bottom':
        this.popconfirm.style.left = wrapperLeft + wrapperWidth / 2 + 'px';
        this.popconfirm.style.bottom = document.body.offsetHeight - wrapperTop - wrapperHeight + 'px';
        break;
      case 'left':
        this.popconfirm.style.left = wrapperLeft + 'px';
        this.popconfirm.style.top = wrapperTop + wrapperHeight / 2 + 'px';
        break;
      case 'right':
        this.popconfirm.style.left = 'auto';
        this.popconfirm.style.right = document.body.offsetWidth - wrapperLeft - wrapperWidth + 'px';
        this.popconfirm.style.top = wrapperTop + wrapperHeight / 2 + 'px';
        break;
      default:
        this.popconfirm.style.right = document.body.offsetWidth - wrapperLeft - wrapperWidth - 4 + 'px';
        this.popconfirm.style.top = wrapperTop + wrapperHeight / 2 + 'px';
        break;
    }
  }

  createPopconfirm() {
    const {direction, title, content, visible} = this.props;
    const contentHTML = renderToString(content);  //将react对象转换为HTML字符串

    this.popconfirm = document.createElement('div');
    this.popconfirm.className = `popover  popover-placement-${direction} popover-action`;
    this.popconfirm.style = `${visible ? 'display: block' : 'display: none'}`;
    this.popconfirm.onclick = function (e) {
      e.stopPropagation();
    };

    this.popconfirm.innerHTML = `<div class=popover-content>
      <div class=popover-arrow></div>
      <div class="popover-inner">
        ${title && `<div class="popover-title">${title}</div>`}
        <div class="popover-inner-content">
          ${contentHTML}
        </div>
      </div>
    </div>`;
    document.body.appendChild(this.popconfirm);
    this.setPosition();
  }

  componentDidMount() {
    this.createPopconfirm();
    document.body.addEventListener("click", () => {
      this.hide();
    });
  }

  show = () => {
    this.popconfirm.style.display = "block";
  };
  hide = () => {
    this.popconfirm.style.display = "none";
  };

  render() {
    return (
      <div
        ref={(dom) => this.wrapper = dom}
        onClick={this.show}
        className="popover-wrapper"
      >
        {this.props.children}
      </div>
    );
  }
}
