import React from 'react'
import PropTypes from 'prop-types'

export default class SearchInput extends React.Component {

  static propTypes = {
    dataSource: PropTypes.array,
    placeholder: PropTypes.string,
    defaultValue: PropTypes.any,
    value: PropTypes.any,
    onChange: PropTypes.func,
    onSelect: PropTypes.func,
    style: PropTypes.object
  };

  constructor(props) {
    super(props);

    this.value = this.props.defaultValue != undefined ? this.props.defaultValue : "";

    this.state = {
      value: this.value,
      action: false,
    }

    this.onChange = (e) => {
      this.value = e.target.value;
      this.props.onChange(this.value);
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.value !== undefined && nextProps.value != this.value) {
      this.value = nextProps.value;
      this.setState({value: nextProps.value})
    }
  }

  onSelectItem(item) {

    // let preValue = this.value;
    this.value = item.text || item;
    this.setState({value: this.value});
    this.onUnAction();

    // if (this.value !== preValue) {
    //   this.props.onChange && this.props.onChange(this.value);
    // }
    this.props.onChange && this.props.onChange(this.value);
    this.props.onSelect && this.props.onSelect(item);

  }

  onUnAction() {
    this.setState({
      action: false
    })
  }

  onAction(e) {
    if (e.target.tagName.toUpperCase() === "LI"){
      this.setState({
        action: false
      })
    } else {
      this.setState({
        action: true
      })
    }
  }


  render() {

    let {dataSource, style, placeholder} = this.props;
    let {action} = this.state;


    let className = action ? "select action" : "select";

    return (
      <div className={className} onMouseLeave={this.onUnAction.bind(this)} onClick={this.onAction.bind(this)}
           style={{...style}}>
        <input ref="input" style={{border: 0, width: "100%"}} placeholder={placeholder} onFocus={this.onChange}
               onChange={this.onChange} value={this.value}/>
        {
          dataSource && dataSource.length > 0 ?
            <div style={{paddingTop: 10, position: "absolute", marginLeft: -1, width: "100%", left: 0}}>
              <div className="select-dropdown" style={{marginTop: 0}}>
                <ul>
                  {
                    dataSource.map((item, index) => {
                      return <li onClick={this.onSelectItem.bind(this, item)} key={index}>{item.text || item}</li>
                    })
                  }
                </ul>
              </div>
            </div> : null
        }
      </div>
    )
  }

}
