import React from 'react'
import PropTypes from 'prop-types'

export default class Select extends React.Component {

  static propTypes = {
    dataSource:PropTypes.array.isRequired,
    placeholder:PropTypes.string,
    defaultValue:PropTypes.any,
    value:PropTypes.any,
    disabled:PropTypes.bool,
    onChange:PropTypes.func,
    style:PropTypes.object,
    showSearch: PropTypes.bool
  };

  constructor(props) {
    super(props);

    this.value = this.props.value != undefined ? this.props.value : this.props.defaultValue;

    this.state = {
      defaultValue:this.props.defaultValue,
      value:this.value,
      action:false,
      dataSource: this.props.dataSource || [],
      isSearch: false,
      searchValue: ''
    }
  }

  componentWillReceiveProps(nextProps) {
    if(nextProps.value !== undefined && nextProps.value != this.state.value){
      this.value = nextProps.value;
      this.setState({value:nextProps.value})
    }
  }

  onSelectItem(item){

    let preValue = this.value;
    this.value = item.value;
    this.setState({value:item.value, isSearch: false});
    this.onUnAction();

    if(item.value !== preValue){
      this.props.onChange && this.props.onChange(item.value, item);
    }

  }

  onUnAction(){
    this.setState({
      isSearch: false,
      action:false,
      searchValue: '',
      dataSource: this.props.dataSource
    })
  }

  onAction(e){
    this.setState({
      action:!this.state.action
    })
  }
  searchChange = (e) => {
    let dataSourceSearch = this.props.dataSource
    if (dataSourceSearch) {
      this.setState({
        isSearch: true,
        searchValue: e.target.value,
        dataSource: dataSourceSearch.filter(item => {
          return item.text.indexOf(e.target.value) > -1
        })
      })
    } else {
      this.setState({
        isSearch: false,
        searchValue: '',
        dataSource: this.props.dataSourceSearch
      })
    }
  }

  render(){

    let {disabled, placeholder, defaultValue, style, showSearch} = this.props;
    let {action , value, dataSource=[], isSearch, searchValue} = this.state;
    let selectItem = dataSource.filter((item)=>{return item.value == value});
    let showText = selectItem && selectItem[0] ? selectItem[0].text : placeholder;

    disabled = !disabled ? ((dataSource.length == 0 && !isSearch) ? true : disabled) : disabled

    let className = disabled ? "select select-disabled" : action ? "select action" : "select";
    return (
        <div className={className} onMouseLeave={this.onUnAction.bind(this)} onClick={this.onAction.bind(this)} style={{...style}}  >
          <div className="select-inner" style={{ opacity: isSearch?'0':'1' }}>{showText}</div>
          <span className="icon-select"></span>
          {(showSearch && !disabled)?<div style={{ position: "absolute", top: 0, left: 0, right: 0, padding: 8 }}>
            <input value={ searchValue } onChange={ this.searchChange } style={{ width: "100%", height: "100%", fontSize: "14px", color: "#666", border: "none", background: "transparent" }}/>
          </div>:''}
          <div style={{paddingTop:10,position:"absolute",marginLeft:-1, width:"100%", left:0}}>
            <div className="select-dropdown" style={{marginTop:0}} >
            <ul>
              {
                dataSource.map((item, index)=>{
                  return <li onClick={this.onSelectItem.bind(this, item)} className={item.value == value? "item-action" : ""} key={index} >{item.text}</li>
                })
              }
            </ul>
            </div>
          </div>
        </div>
    )
  }

}
