import React from 'react'
import PropTypes from 'prop-types'

export default class Table extends React.Component {
  
  static propTypes = {
    dataSource:PropTypes.array.isRequired,
    columns:PropTypes.array.isRequired,
    onRowClick:PropTypes.func, //行点击
    isIndex:PropTypes.bool, //显示行号
    loading:PropTypes.bool, //是否记载中
    pageSize:PropTypes.number, //最大显示的行数
    rowKey:PropTypes.string,
    /**
      {
        type:"checkbox|radio" //单选多选，默认多选
        selectedRowKeys:"" //默认选中的项目
        onChange:(selectedRowKeys, selectedRows)=>{}
      }
    **/
    rowSelection:PropTypes.any//配置列表项可选
  };

  constructor(props) {
    super(props);
    this.state = {
      selectAll:false,
      rowSelection:props.rowSelection
    }
  }

  renderTD(item, columnsKeys, columnMaps, trIndex){
    return columnsKeys.map((key, index)=>{
      return <td key={index}>{columnMaps[key] && columnMaps[key].render ? columnMaps[key].render(item[key], item, trIndex) : item[key]}</td>
    });
  }
  onRowClick(rowData, index){

    let {rowSelection} = this.state;
    if(rowSelection){//如果列表需要选择，则选中或者取消选中
      rowSelection = Object.assign({},rowSelection);
      let key = rowData.key;
      rowSelection.selectedRowKeys = rowSelection.selectedRowKeys || [];
      let inIndex = rowSelection.selectedRowKeys.indexOf(key);
      if(inIndex === -1){//未有
        rowSelection.selectedRowKeys.push(key)
      }else{
        rowSelection.selectedRowKeys.splice(inIndex, 1);
        
      }

      this.setState({
        selectAll:rowSelection.selectedRowKeys.length == this.getDataSource().length, 
        rowSelection:rowSelection
      });
      this.onSelectRowChange(rowSelection);
    }

    this.props.onRowClick && this.props.onRowClick(rowData, index);
  }

  onSelectRowChange(rowSelection){
    let {dataSource} = this.props;
    let rows = dataSource.filter((item)=>{ return  rowSelection.selectedRowKeys.indexOf(item.key) != -1;})
    this.props.rowSelection && this.props.rowSelection.onChange && this.props.rowSelection.onChange(rowSelection.selectedRowKeys, rows)
  }

  reset(){
    let {rowSelection} = this.state;
    rowSelection.selectedRowKeys = [];
    this.setState({
      selectAll:false,
      rowSelection:rowSelection
    });
    this.onSelectRowChange(rowSelection)
  }

  //添加选择头
  addSelectionCloumns(columns){
    let {rowSelection} = this.state;
    let selectedRowKeys = rowSelection.selectedRowKeys || [];
    return [
      {
        title:"选择项", 
        key:"$_select", 
        render:(text, item)=>{ return <input type="checkbox" checked={selectedRowKeys.indexOf(item.key) != -1} onChange={()=>{}}  /> }
      }
    ].concat(columns);
  }

  getDataSource(){
    //限制最大显示行数
    let {pageSize, dataSource} = this.props;

    if(pageSize){
      dataSource = dataSource.slice(0, pageSize)
    }

    return dataSource;

  }
  //渲染自己定义的头 比如全选按钮
  renderTh(item){

    if(item.key === "$_select"){
      //如果当前列表需要选中行选中，渲染一个全选checkbox
      let {selectAll, rowSelection} = this.state;
      let {dataSource, pageSize} = this.props;
      return <input 
        type="checkbox" 
        checked={selectAll} 
        onChange={()=>{
          //全选
          rowSelection.selectedRowKeys = selectAll ? [] : dataSource.slice(0, pageSize).map((item)=>{ return item.key; });
          this.setState({
            selectAll:!selectAll,
            rowSelection:rowSelection
          });
          this.onSelectRowChange(rowSelection)
        }}  
      />
    }

    return item.title;
  }

  renderNoData(dataSource, columns){
    let {loading} = this.props;
    let context = null;

    if(dataSource.length){
      return null
    }
    
    if(dataSource.length == 0 && loading){
      context = (<div className="loading-wrapper">
          <span className="loading"></span>
          <span>正在加载中...</span>
      </div>)
    }else{
      context = "暂无数据";
    }

    return <tr><td colSpan={columns.length+1} style={{textAlign:"center"}} >{context}</td></tr>
  }

  render(){

    let {dataSource, columns, isIndex, pageSize, rowSelection, rowKey} = this.props;

    //如果要显示行号
    if(isIndex){
      columns = [{title:"#",key:"_index"}].concat(columns);
      dataSource = dataSource.map((item, index)=>{ item._index = index+1; return item });
    }

    //列表可选择
    if(rowSelection){
      columns = this.addSelectionCloumns(columns);
    }

    

    dataSource = this.getDataSource();


    let columnMaps = {};
    let columnsKeys = columns.map((item)=>{
      columnMaps[item.key] = item;
      return item.key;
    });


    return (
      <table className="table">
        <thead>
           <tr>
          {
            columns.map((item, index)=>{
              return <th key={index}>{this.renderTh(item)}</th>
            })
          }
          </tr>
        </thead>
        <tbody>
          {
            dataSource.map((item, index)=>{
              return (
                <tr 
                  key={item[rowKey] || item.key || index}
                  className={rowSelection && rowSelection.selectedRowKeys.indexOf(item.key) != -1 ? "active" : ""} 
                  onClick={this.onRowClick.bind(this, item, index)} 
                >
                  {this.renderTD(item, columnsKeys, columnMaps, index)}
                </tr>
              )
            })
          }
          {
            this.renderNoData(dataSource, columns)
          }
        </tbody>
      </table>
    )
  }

}


