import React from 'react'
import PropTypes from 'prop-types'

export default class Input extends React.Component {

	static propTypes = {
		type:PropTypes.string,
		disabled:PropTypes.bool,
		defaultValue:PropTypes.string,
		onChange:PropTypes.func,
		// width:PropTypes.string,
		// height:PropTypes.string,
		placeholder:PropTypes.string,
		autoClose:PropTypes.bool
	};

	constructor(props) {
		super(props);
		this.value = props.defaultValue || "";
		let times = this.value.split(":");
		this.state={
			action:false,
			select:{
				hour:times[0]||"",
				minute:times[1]||""
			},
			value:this.value
		}
		this.length = {
			hour:24,
			minute:60
		}
	}

	componentWillReceiveProps(props){
		if(props.value !== undefined && props.value!=this.value){
			this.value = props.value;
			let times = this.value.split(":");
			this.setState({
				select:{
					hour:times[0]||"",
					minute:times[1]||""
				},
				value: this.value
			})
		}
	}

	getTime(type){
		var {select} = this.state;
		let time = [];
		for (let i=0; i<this.length[type]; i++){
			i = i<10?'0'+i:i
			time.push(<li key={i} value={i} className={select[type]==i?"item-action":""} onMouseDown={(e)=>{
				!(this.props.autoClose&&type=='minute')&&e.preventDefault();
				this.onSelect(type,i);
			}}>{i}</li>)
		}
		return (<div className="time-picker-select">
			<ul>
				{time}
			</ul>
		</div>)
	}

	onSelect(type,item){
		let {select} = this.state;
		select[type] = item;
		this.value = (select['hour']||'00')+":"+(select['minute']||"00");
		this.setState({
			select,
			value:this.value
		})
	}

	onFocus (){
		this.setState({
			action:true
		})
	}
	onBlur(){
		this.setState({
			action:false
		})
		this.onChange()
	}

	onChange(){
		this.props.onChange && this.props.onChange(this.value)
	}

	render(){
		let {action,value} = this.state;
		let {disabled,placeholder} = this.props;
		return (
			<div className={"time-picker "+(action?"time-picker-action ":"")+(disabled?"time-picker-disabled":"")}>
				<div className="time-picker-input-wrapper ">
					<input type="" name="" className="time-picker-input" ref="input" disabled={disabled} value={value} onChange={(e)=>{
						this.onChange();
					}} placeholder={placeholder}
					       onFocus={this.onFocus.bind(this)}
					       onBlur={this.onBlur.bind(this)}
					/>
						<span className="icon-time-picker"></span>
				</div>
				<div className="time-picker-combobox-wrapper">
					{this.getTime('hour')}
					{this.getTime('minute')}
				</div>
			</div>
			)
	}
}
