import React from 'react'
import PropTypes from 'prop-types'

const directionSet = {
  top: true,
  bottom: true,
  left: true,
  right: true,
};

export default class Tooltip extends React.Component {

  static defaultProps = {
    direction: 'right',
  };

  static propTypes = {
    title: PropTypes.string.isRequired,
    children: PropTypes.element.isRequired,
    direction: PropTypes.string,
    onEnter: PropTypes.func,
    onLeave: PropTypes.func
  };

  tooltip = null;
  wrapper = null;

  getDirection() {
    const {direction} = this.props;
    return directionSet[direction] ? direction : 'right';
  }

  getElementLeft(element) {
    let actualLeft = element.offsetLeft;
    let current = element.offsetParent;
    while (current !== null) {
      actualLeft += current.offsetLeft;
      current = current.offsetParent;
    }
    return actualLeft;
  }

  getElementTop(element) {
    let actualTop = element.offsetTop;
    let current = element.offsetParent;
    while (current !== null) {
      actualTop += current.offsetTop;
      current = current.offsetParent;
    }
    return actualTop;
  }

  setPosition() {
    const wrapperStyle = getComputedStyle(this.wrapper, null);
    const tooltipStyle = getComputedStyle(this.tooltip, null);

    const wrapperWidth = parseFloat(wrapperStyle.width);
    const wrapperHeight = parseFloat(wrapperStyle.height);
    const tooltipWidth = parseFloat(tooltipStyle.width);
    const tooltipHeight = parseFloat(tooltipStyle.height);

    const wrapperTop = this.getElementTop(this.wrapper);
    const wrapperLeft = this.getElementLeft(this.wrapper);


    switch (this.props.direction) {
      case 'top':
        this.tooltip.style.left = wrapperLeft + wrapperWidth / 2 + 'px';
        this.tooltip.style.top = wrapperTop + 'px';
        break;
      case 'bottom':
        this.tooltip.style.left = wrapperLeft + wrapperWidth / 2 + 'px';
        this.tooltip.style.bottom = document.body.offsetHeight - wrapperTop - wrapperHeight + 'px';
        break;
      case 'left':
        this.tooltip.style.left = wrapperLeft + 'px';
        this.tooltip.style.top = wrapperTop + wrapperHeight / 2 + 'px';
        break;
      case 'right':
        this.tooltip.style.right = document.body.offsetWidth - wrapperLeft - wrapperWidth - 4 + 'px';
        this.tooltip.style.top = wrapperTop + wrapperHeight / 2 + 'px';
        break;
      default:
        this.tooltip.style.right = document.body.offsetWidth - wrapperLeft - wrapperWidth - 4 + 'px';
        this.tooltip.style.top = wrapperTop + wrapperHeight / 2 + 'px';
        break;
    }
  }

  //在body元素的末尾创建tooltip悬浮框
  createToolTip() {
    this.tooltip = document.createElement('div');
    this.tooltip.className = `tooltip tooltip-action tooltip-placement-${this.getDirection()}`;
    this.tooltip.innerHTML = `<div class="tooltip-arrow"></div>
<div class="tooltip-inner" style="white-space: normal; word-break:normal;">${this.props.title}</div>`;
    document.body.appendChild(this.tooltip);
    this.setPosition();
  }

  //移除之前创建的tooltip悬浮框
  removeToolTip() {
    this.tooltip && document.body.removeChild(this.tooltip);
    this.tooltip = null;
  }

  mouseEnter = () => {
    this.createToolTip();
    this.props.onEnter && this.props.onEnter();
  };

  mouseLeave = () => {
    this.removeToolTip();
    this.props.onLeave && this.props.onLeave();
  };

  render() {
    const {children} = this.props;

    return (
      <div
        ref={(dom) => this.wrapper = dom}
        className="tooltip-wrapper"
        onMouseEnter={this.mouseEnter}
        onMouseLeave={this.mouseLeave}>
        {children}
      </div>
    );
  }
}
