import React, {Component} from 'react'
import PropTypes from 'prop-types'
// import {Button} from './index'
import Button from './Button'
import message from './Message'
import Progress from './Progress'

export default class Upload extends Component {

  static defaultProps = {
    desc: "",
    accept: "",
    size: {},
    disabled: false,
    withCredentials: false,
    fileList: [],
    listType: 'text',
    showProgress: false
  }

  static propTypes = {
    action: PropTypes.string.isRequired,
    desc: PropTypes.string,
    onLoad: PropTypes.func,
    onProgress: PropTypes.func,
    showProgress: PropTypes.bool,
    onError: PropTypes.func,
    accept: PropTypes.string,
    customRequest: PropTypes.func,
    disabled: PropTypes.bool,
    withCredentials: PropTypes.bool,
    fileList: PropTypes.array,
    removeFile: PropTypes.func,
    listType: PropTypes.string
  }

  constructor(props) {
    super(props)
    this.state = {process:0};
    this.elementId = props.id || "fileInput_" + Math.floor(Math.random() * 100000000000);

    this.fileInputChange = (e) => {
      this._fileInputChange(e)
    }

    this.uploadClick = (e) => {
      this._uploadClick(e)
    }
  }

  reset() {
    document.getElementById(this.elementId).value = "";
  }

  _uploadClick() {
    document.getElementById(this.elementId).click()
  }

  _fileInputChange(e) {
    let {customRequest, accept} = this.props;
    let files = e.currentTarget.files || [];
    if (files.length === 0) {
      return false
    }
    if (files[0].type.indexOf(accept) > -1) {
      if (customRequest && typeof customRequest === "function") {
        customRequest(e)
      } else {
        this.uploadFile(files);
      }
    } else {
      message.error(`请选择${accept}文件`)
    }
    this.reset()

  }

  tryToJSON(str) {
    if (typeof str === 'string') {
      try {
        return eval(`evalJSONString(${str})`)
      } catch (e) {
      }

      try {
        return JSON.parse(str)
      } catch (e) {
      }

      return str;

    }
  }

  uploadFile(files, index = 0) {
    let {action, onLoad, onProgress, onError, withCredentials, accept, size} = this.props;
    let self = this;

    let formData = new FormData();
    let file = files[index];
    if (size.maxSize && file.size > size.maxSize) {
      message.error("文件超出大小限制")
    }
    formData.append("file", file);

    let xhr = new XMLHttpRequest();
    xhr.upload.onprogress = onProgress || function (e) {
        self.setState({process: parseInt(e.loaded / e.total * 100)})
      };

    xhr.onload = (e) => {
      let current = e.currentTarget;
      self.setState({upload: false});
      if (Math.floor(current.status / 100) === 2 || Math.floor(current.status / 100) === 3) {
        let res = this.tryToJSON(current.responseText)
        onLoad && onLoad(res, e)
      } else {
        onError && onError({msg: "请求失败", resultCode: current.status}, e)
      }
    };

    xhr.onerror = onError || function (e) {
        console.error("onerror", e)
        self.setState({process:0,upload: true});
      }

    xhr.open("POST", action, true);
    xhr.withCredentials = withCredentials;

    if (accept === "image" && (size.width || size.height || action === "base64")) {
      let reader = new FileReader();
      reader.onload = function (e) {
        let data = e.target.result;
        //加载图片获取图片真实宽度和高度
        let image = new Image();
        image.onload = function () {
          let width = image.width;
          let height = image.height;
          if (size.width && size.width != width) {
            message.error("图片宽度不符合要求")
            return false
          }
          if (size.height && size.height != height) {
            message.error("图片高度不符合要求")
            return false
          }
          if (action === "base64") {
            self.value = data;
            onLoad && onLoad(data, e)
          } else {
            xhr.send(formData);
            self.setState({process:0,upload: true});
          }
        };
        image.src = data;
      };
      reader.readAsDataURL(file);
    } else {
      xhr.send(formData);
      self.setState({process:0,upload: true});
    }

  }

  render() {
    let {desc, disabled, children, fileList, removeFile, listType, showProgress} = this.props;
    fileList = fileList || [];

    let {upload, process} = this.state;

    if (!children) children = "上传文件";

    return <div className="upload-wrapper upload-file">
      <div className="upload">
        <input type="file" className="file-input" id={this.elementId} onChange={this.fileInputChange}/>
        <Button disabled={disabled} onClick={this.uploadClick} secondary={true}>
          <span className="icon-antion-hover-upload"/>&nbsp;{children}</Button>
      </div>
      {desc ? <div className="explain">{desc}</div> : null}
      {(upload && showProgress) ? <Progress progress={process} type={process == 100 ? "success" : ""}/> : null}
      {
        (listType === 'text' && fileList.length > 0) ? <div className="d-list">
          {
            fileList.map((file, i) => {
              let fileName = file;
              let href = ""
              if (typeof file === "object") {
                fileName = file.name
                href = file.href || ""
              }
              return <div className="upload-list-item" key={i}>
                <span className="icon-antion-clip"/>
                {href ?
                  <a className="txt-link" style={{paddingRight: "40px"}} href={href}
                     target="_blank">{fileName}</a> :
                  <a className="txt-link" style={{paddingRight: "40px"}}>{fileName}</a>}
                {(removeFile && typeof removeFile === 'function') ?
                  <span className="icon-antion-hover-remove" onClick={() => {
                    removeFile(file)
                  }}/> : null}
              </div>
            })
          }
        </div> : null
      }
      {
        (listType === 'picture' && fileList.length > 0 ) ? <div style={{marginTop: 10}}>
          {
            fileList.map((file, i) => {
              let href = file;
              if (typeof file === "object") {
                href = file.href || ""
              }
              return <div key={i} className="upload-wrapper upload-picture-card upload-picture-card-after">
                <img src={href}/>
                <div className="upload-picture-icon">
                  <a className="upload-picture-item-thumbnail" href={href} target="_blank">
                    <span className="icon-antion-see"/>
                  </a>
                  <span className="icon-antion-delete" onClick={() => {
                    removeFile(file)
                  }}/>
                </div>
              </div>
            })
          }
        </div> : null
      }
    </div>
  }
}