
import React from 'react'
import {render} from 'react-dom'
import SyntaxHighlighter from 'react-syntax-highlighter';
import { docco } from 'react-syntax-highlighter/dist/styles';

import {Router, Route, IndexRoute, IndexRedirect, browserHistory,hashHistory,useRouterHistory,Link } from 'react-router'

import {createHashHistory} from 'history'
//{queryKey: false}
const history = useRouterHistory(createHashHistory)()

import PIROS from './index';
import IndexDOC from './doc/Index';
import LayoutDOC from './doc/Layout'
import MenuDOC from './doc/Menu'
import Panel from './doc/Panel'
import Button from './doc/Button'
import Table from './doc/Table'
import Tooltip from './doc/Tooltip'
import Popconfirm from './doc/Popconfirm'
import Message from './doc/Message'
import Input from './doc/Input'
import Switch from './doc/Switch'
import Radio from './doc/Radio'
import CheckBox from './doc/CheckBox'
import InputNumber from './doc/InputNumber'
import Icon from './doc/Icon'
import Tag from './doc/Tag'
import Badge from './doc/Badge'
import Tabs from './doc/Tabs'
import Progress from './doc/Progress'
import Select from './doc/Select'
import SearchInput from './doc/SearchInput'
import DatePicker from './doc/DatePicker'

import Pagination from './doc/Pagination'
import Modal from './doc/Modal'
import TimerPicker from './doc/TimerPicker'
import Alert from './doc/Alert'
import Upload from './doc/Upload'

let components = [
  {
    title:"General",
    key:"General",
    list:[
      {
        title:"Index",
        component:IndexDOC,
        key:"Index",
        href:"/Index"
      },
      {
        title:"Layout",
        component:LayoutDOC,
        key:"Layout",
        href:"/Layout"
      },
      
      {
        title:"Panel",
        component:Panel,
        key:"Panel",
        href:"/Panel"
      },
      {
        title:"Button",
        component:Button,
        key:"Button",
        href:"/Button"
      }
      
      
    ]
  },
  {
    title:"Navigation",
    key:"Navigation",
    list:[
      {
        title:"Pagination",
        component:Pagination,
        key:"Pagination",
        href:"/Pagination"
      },
      {
        title:"Menu",
        component:MenuDOC,
        key:"Menu",
        href:"/Menu"
      },
      {
        title:"Tabs",
        component:Tabs,
        key:"tabs",
        href:"/Tabs"
      }
    ]
  },
  {
    title:"Data Entry",
    key:"DataEntry",
    list:[
      {
        title:"DatePicker",
	      component:DatePicker,
        href:"/DatePicker"
      },
      {
        title:"Select",
        component:Select,
        key:"Select",
        href:"/Select"
      },
      {
        title:"SearchInput",
        component: SearchInput,
        key:"SearchInput",
        href:"/SearchInput"
      },
	    {
	    	title:"input",
		    component:Input,
        key:"Input",
		    href:"/Input"
	    },
	    {
		    title:"Switch",
		    component:Switch,
        key:"Switch",
		    href:"/Switch"
	    },
	    {
	    	title:"Radio",
		    component:Radio,
        key:"Radio",
		    href:"/Radio"
      },
      {
        title:"CheckBox",
        component:CheckBox,
        key:"CheckBox",
        href:"/CheckBox"
	    },
	    {
		    title:"InputNumber",
		    component:InputNumber,
        key:"InputNumber",
		    href:"/InputNumber"
	    },
	    {
		    title:"TimerPicker",
		    component:TimerPicker,
		    key:"TimerPicker",
		    href:"/TimerPicker"
	    },
      {
        title:"Upload 上传",
        component:Upload,
        key:"Upload",
        href:"/Upload"
      }
    ]
  },
  {
    title:"Data Display",
    key:"DataDisplay",
    list:[
      {
        title:"Table",
        component:Table,
        key:"Table",
        href:"/Table"
      },
      {
        title:"Collapse",
        key:"Collapse",
        href:"/Collapse"
      },
      
      {
        title:"Tag 标签",
        component:Tag,
        key:"Tag",
        href:"/Tag"
      },
      {
        title:"Badge 徽标",
        component:Badge,
        href:"/Badge",
        key:"Badge",
      },
      {

        title:"Tooltip",
        href:"/Tooltip",
        component:Tooltip,
        key:"Tooltip",
      }
    ]
  },
  {
    title:"Feedback",
    key:"feedback",
    list:[
      {
        title:"Alert ",
        component:Alert,
        key:"Alert",
        href:"/Alert"
      },
      {
        title:"Modal",
        component:Modal,
        key:"Modal",
        href:"/Modal"
      },
      {
        title:"Message",
	      component:Message,
        key:"Message",
        href:"/Message"
      },
      {
        title:"Progress",
        component: Progress,
        key:"Progress",
        href:"/Progress"
      },
      {
        title:"Popconfirm",
        href:"/Popconfirm",
        component: Popconfirm,
        key:"Popconfirm",
      },
      {
        title:"Icon font",
        component:Icon,
        key:"Icon",
        href:"/Icon"
      }
    ]
  }
  
];


class Layout extends React.Component{

  constructor(props) {
    super(props);
    this.state = {title:"PIROS React Component"}
    this.menuDataSource = components.map((item)=>{
      let menu = Object.assign({}, item);
      if(menu.href){
        menu.href = "#"+menu.href;
      }

      if(menu.list){
        menu.list = menu.list.map((item)=>{
          if(item.href){
            item.href = "#"+item.href;
          }
          return item;
        })
      }

      return menu;
    });
  }

  render(){
    let Layout = PIROS.Layout;

    return <Layout title="title" >
    <Layout.Side>
      <PIROS.Menu 
        dataSource={this.menuDataSource}
        defaultOpenKeys={["General","Navigation", "feedback", "DataDisplay", "DataEntry"]}
        defaultSelectedKeys={["Index"]}
        onSelect={(menu)=>{ console.log(menu) }}
      />
    </Layout.Side>
    <Layout.Context>
      {this.props.children}
    </Layout.Context>
  </Layout>
  }
}


function contextDoc(componentDoc){
  return (
    class Context extends React.Component{
      constructor(props) {
        super(props)
        this.state = {}
      }

      render(){
        return (
          <div>
            <p className="con-title">{componentDoc.title || "PIROS React Component"}</p>
            <PIROS.Panel title="效果" >
              {componentDoc.example.call(this)}
            </PIROS.Panel>
            <PIROS.Panel title="参数说明" >
              {componentDoc.params()}
            </PIROS.Panel>
            <PIROS.Panel title="Code" >
              <SyntaxHighlighter language='javascript' style={docco} >
                {componentDoc.code()}
              </SyntaxHighlighter>
            </PIROS.Panel>
          </div>
        ) 
      }
    }
  )
}

export default class App extends React.Component{

    render() {
      let routers = []
      components.map((item)=>{
        item.list && item.list.map((i)=>{
          i.component && routers.push(i)
        })
      });
      return (
        <Router history={history}>
          <Route path="/" component={Layout}>
            <IndexRoute component={contextDoc(routers[0].component)}  />
            {
              routers.map((item)=>{
                return <Route path={item.href} key={item.href} component={contextDoc(item.component)} />
              })
            }
          </Route>
        </Router>
      )
    }
}

render(<App />, document.getElementById("root"))


